import { __assign, __extends, __read, __spread } from "tslib";
import { AbstractOp } from './AbstractOp';
import { find, isArrayReference, formatJsonPointer } from '../../json-pointer';
import { isTextNode, isElementNode } from '../util';
var OpMerge = (function (_super) {
    __extends(OpMerge, _super);
    function OpMerge(path, pos, props) {
        var _this = _super.call(this, 'merge', path) || this;
        _this.pos = pos;
        _this.props = props;
        return _this;
    }
    OpMerge.prototype.apply = function (doc) {
        var ref = find(doc, this.path);
        if (!isArrayReference(ref))
            throw new Error('INVALID_TARGET');
        if (ref.key <= 0)
            throw new Error('INVALID_KEY');
        var one = ref.obj[ref.key - 1];
        var two = ref.obj[ref.key];
        var merged = this.merge(one, two);
        ref.obj[ref.key - 1] = merged;
        ref.obj.splice(ref.key, 1);
        return { doc: doc, old: [one, two] };
    };
    OpMerge.prototype.merge = function (one, two) {
        if (typeof one === 'string' && typeof two === 'string')
            return one + two;
        if (typeof one === 'number' && typeof two === 'number')
            return one + two;
        if (isTextNode(one) && isTextNode(two))
            return __assign(__assign(__assign({}, one), two), { text: one.text + two.text });
        if (isElementNode(one) && isElementNode(two))
            return __assign(__assign(__assign({}, one), two), { children: __spread(one.children, two.children) });
        return [one, two];
    };
    OpMerge.prototype.toJson = function () {
        var op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            pos: this.pos,
        };
        if (this.props)
            op.props = this.props;
        return op;
    };
    OpMerge.prototype.toPacked = function () {
        var packed = [11, this.path, { i: this.pos }];
        if (this.props)
            packed[2].p = this.props;
        return packed;
    };
    return OpMerge;
}(AbstractOp));
export { OpMerge };
