import { __extends } from "tslib";
import { AbstractOp } from './AbstractOp';
import { find, isObjectReference, isArrayReference, formatJsonPointer } from '../../json-pointer';
var OpRemove = (function (_super) {
    __extends(OpRemove, _super);
    function OpRemove(path, oldValue) {
        var _this = _super.call(this, 'remove', path) || this;
        _this.oldValue = oldValue;
        return _this;
    }
    OpRemove.prototype.apply = function (doc) {
        var ref = find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        if (isObjectReference(ref))
            delete ref.obj[ref.key];
        else if (isArrayReference(ref)) {
            if (ref.val !== undefined)
                ref.obj.splice(ref.key, 1);
        }
        else
            doc = null;
        return { doc: doc, old: ref.val };
    };
    OpRemove.prototype.toJson = function () {
        var json = {
            op: this.op,
            path: formatJsonPointer(this.path),
        };
        if (this.oldValue !== undefined)
            json.oldValue = this.oldValue;
        return json;
    };
    OpRemove.prototype.toPacked = function () {
        var packed = this.oldValue !== undefined ? [1, this.path] : [1, this.path, { o: this.oldValue }];
        return packed;
    };
    return OpRemove;
}(AbstractOp));
export { OpRemove };
