import { __extends, __read, __values } from "tslib";
import { AbstractOp } from './AbstractOp';
import { find, isArrayReference, isObjectReference, formatJsonPointer } from '../../json-pointer';
var isArray = Array.isArray;
var OpExtend = (function (_super) {
    __extends(OpExtend, _super);
    function OpExtend(path, props, deleteNull) {
        var _this = _super.call(this, 'extend', path) || this;
        _this.props = props;
        _this.deleteNull = deleteNull;
        return _this;
    }
    OpExtend.prototype.apply = function (doc) {
        var ref = find(doc, this.path);
        if (isArrayReference(ref)) {
            if (ref.val !== undefined) {
                ref.obj[ref.key] = this.extend(ref.val);
            }
        }
        else if (isObjectReference(ref)) {
            ref.obj[ref.key] = this.extend(ref.val);
        }
        else {
            doc = this.extend(doc);
        }
        return { doc: doc };
    };
    OpExtend.prototype.extend = function (value) {
        var e_1, _a;
        if (isArray(value))
            return value;
        if (typeof value !== 'object')
            return value;
        if (!value)
            return value;
        try {
            for (var _b = __values(Object.entries(this.props)), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = __read(_c.value, 2), key = _d[0], v = _d[1];
                if (key === '__proto__')
                    throw new Error('NO_PROTO');
                if (v === null && this.deleteNull) {
                    delete value[key];
                    continue;
                }
                value[key] = v;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return value;
    };
    OpExtend.prototype.toJson = function () {
        var op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            props: this.props,
        };
        if (this.deleteNull)
            op.deleteNull = this.deleteNull;
        return op;
    };
    OpExtend.prototype.toPacked = function () {
        var packed = [12, this.path, { p: this.props }];
        if (this.deleteNull)
            packed[2].d = 1;
        return packed;
    };
    return OpExtend;
}(AbstractOp));
export { OpExtend };
