import { __assign, __extends } from "tslib";
import { AbstractOp } from './AbstractOp';
import { find, isObjectReference, isArrayReference, formatJsonPointer } from '../../json-pointer';
import { isTextNode, isElementNode } from '../util';
var OpSplit = (function (_super) {
    __extends(OpSplit, _super);
    function OpSplit(path, pos, props) {
        var _this = _super.call(this, 'split', path) || this;
        _this.pos = pos;
        _this.props = props;
        return _this;
    }
    OpSplit.prototype.apply = function (doc) {
        var ref = find(doc, this.path);
        if (ref.val === undefined)
            throw new Error('NOT_FOUND');
        var tuple = this.split(ref.val);
        if (isObjectReference(ref))
            ref.obj[ref.key] = tuple;
        else if (isArrayReference(ref)) {
            ref.obj[ref.key] = tuple[0];
            ref.obj.splice(ref.key + 1, 0, tuple[1]);
        }
        else
            doc = tuple;
        return { doc: doc, old: ref.val };
    };
    OpSplit.prototype.split = function (node) {
        if (typeof node === 'string') {
            var _a = this, pos = _a.pos, props = _a.props;
            var before = node.slice(0, pos);
            var after = node.slice(pos);
            if (!props)
                return [before, after];
            var textNodes = [
                __assign(__assign({}, props), { text: before }),
                __assign(__assign({}, props), { text: after }),
            ];
            return textNodes;
        }
        else if (isTextNode(node)) {
            var _b = this, pos = _b.pos, props = _b.props;
            var before = node.text.slice(0, pos);
            var after = node.text.slice(pos);
            var textNodes = [
                __assign(__assign(__assign({}, node), props), { text: before }),
                __assign(__assign(__assign({}, node), props), { text: after }),
            ];
            return textNodes;
        }
        else if (isElementNode(node)) {
            var _c = this, pos = _c.pos, props = _c.props;
            var before = node.children.slice(0, pos);
            var after = node.children.slice(pos);
            var elementNodes = [
                __assign(__assign(__assign({}, node), props), { children: before }),
                __assign(__assign(__assign({}, node), props), { children: after }),
            ];
            return elementNodes;
        }
        else if (typeof node === 'number') {
            var pos = this.pos;
            return [pos, node - pos];
        }
        else
            return [node, node];
    };
    OpSplit.prototype.toJson = function () {
        var op = {
            op: this.op,
            path: formatJsonPointer(this.path),
            pos: this.pos,
        };
        if (this.props)
            op.props = this.props;
        return op;
    };
    OpSplit.prototype.toPacked = function () {
        var packed = [10, this.path, { i: this.pos }];
        if (this.props)
            packed[2].p = this.props;
        return packed;
    };
    return OpSplit;
}(AbstractOp));
export { OpSplit };
