import { __read, __values } from "tslib";
import { from } from 'rxjs';
import { assertId, assertName, isArray, microtask } from './util';
var JsonRxServer = (function () {
    function JsonRxServer(_a) {
        var send = _a.send, call = _a.call, notify = _a.notify, _b = _a.maxActiveSubscriptions, maxActiveSubscriptions = _b === void 0 ? 30 : _b;
        this.active = new Map();
        this.send = send;
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
    }
    JsonRxServer.prototype.sendError = function (id, error) {
        var message = [-1, id, error];
        this.send(message);
    };
    JsonRxServer.prototype.onSubscribe = function (message) {
        var _this = this;
        var _a = __read(message, 3), id = _a[0], name = _a[1], payload = _a[2];
        assertId(id);
        assertName(name);
        try {
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            var observable = from(this.call(name, payload));
            var ref_1 = { buffer: [] };
            var subscription = observable.subscribe(function (data) {
                if (data === undefined)
                    return;
                ref_1.buffer.push(data);
                microtask(function () {
                    var e_1, _a;
                    if (!ref_1.buffer.length)
                        return;
                    try {
                        try {
                            for (var _b = __values(ref_1.buffer), _c = _b.next(); !_c.done; _c = _b.next()) {
                                var payload_1 = _c.value;
                                _this.send([-2, id, payload_1]);
                            }
                        }
                        catch (e_1_1) { e_1 = { error: e_1_1 }; }
                        finally {
                            try {
                                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                            }
                            finally { if (e_1) throw e_1.error; }
                        }
                    }
                    finally {
                        ref_1.buffer = [];
                    }
                });
            }, function (error) {
                var data = error instanceof Error ? { message: error.message } : error;
                var message = [-1, id, data];
                _this.send(message);
            }, function () {
                try {
                    if (!ref_1.buffer.length)
                        return _this.send([0, id]);
                    var last = ref_1.buffer.length - 1;
                    for (var i = 0; i <= last; i++) {
                        var isLast = i === last;
                        var message_1 = isLast ? [0, id, ref_1.buffer[i]] : [-2, id, ref_1.buffer[i]];
                        _this.send(message_1);
                    }
                }
                finally {
                    ref_1.buffer = [];
                }
            });
            this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    };
    JsonRxServer.prototype.onUnsubscribe = function (message) {
        var _a = __read(message, 2), id = _a[1];
        assertId(id);
        var subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    };
    JsonRxServer.prototype.onNotification = function (message) {
        var _a = __read(message, 2), name = _a[0], payload = _a[1];
        assertName(name);
        this.notify(name, payload);
    };
    JsonRxServer.prototype.onMessage = function (message) {
        if (!isArray(message))
            throw new Error('Invalid message');
        var _a = __read(message, 1), one = _a[0];
        if (typeof one === 'string')
            return this.onNotification(message);
        if (one > 0)
            return this.onSubscribe(message);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    };
    return JsonRxServer;
}());
export { JsonRxServer };
